'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.getAttributes = getAttributes;
exports.getLabel = getLabel;
exports.createInputSource = createInputSource;
exports.createInputRich = createInputRich;
exports.createInputFile = createInputFile;
exports.createInputTextarea = createInputTextarea;
exports.createInputLink = createInputLink;
exports.createInputImage = createInputImage;
exports.createInputText = createInputText;
exports.printInput = printInput;

var _sourceAutocomplete = require('./sourceAutocomplete');

var _sourceAutocomplete2 = _interopRequireDefault(_sourceAutocomplete);

var _sourceOption = require('./sourceOption');

var _sourceOption2 = _interopRequireDefault(_sourceOption);

var _ = require('../../../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getAttributes(params) {
  var attributes = '';
  if (params.key != null) attributes += 'id="' + params.key + '" data-id="' + params.key + '"';
  if (params.value != null) attributes += ' value="' + params.value + '"';
  if (params['max-length'] != null) attributes += ' data-maxlength="' + params['max-length'] + '"';
  if (params.reload != null) attributes += ' reload="' + params.reload + '"';
  if (params.order != null) attributes += ' tabIndex="' + params.order + '"';
  if (params.required != null) attributes += ' data-required="' + params.required + '"';
  if (params.display != null) attributes += ' data-display="' + params.display + '"';
  if (params.visible != null) attributes += ' data-visible="' + params.visible + '"';
  if (params.autocomplete != null) attributes += ' data-autocomplete="' + params.autocomplete + '"';
  if (params.placeholder != null) attributes += ' placeholder="' + params.placeholder + '"';
  if (params.thumbs != null) attributes += ' data-size="' + params.thumbs + '"';
  if (params.multiple != null) attributes += ' ' + params.multiple;
  if (params.disabled != null) attributes += ' ' + params.disabled;
  return attributes;
}

function getLabel(params) {
  var desc = params.desc + (params.required ? ' *' : '');
  return '<label class="control-label" for="' + params.key + '" >\n            ' + desc + '\n          </label>';
}

function createInputSource(attributes, inputClass, params) {
  var inputSource = '';
  var lastValues;
  if (params.autocomplete != null && params.autocomplete === 'true') {
    if (params.sourceString.indexOf('http') === 0) lastValues = params.source;else lastValues = JSON.stringify(params.source).replace(/\'/g, '&quote;');
    inputSource += '<div class="autocomplete-result-wrapper">';
    if (params.autocomplete != null && params.autocomplete === 'true' && params.prefill === 'true') {
      inputSource += '<div  class="autocomplete-refresh" value=\'\'\n                            data-autocomplete-refresh="true"\n                            data-autocomplete-refresh-sourcestring="' + params.sourceString + '"\n                            data-autocomplete-refresh-prefill-quantity="' + params['prefill-quantity'] + '"\n                            data-autocomplete-refresh-key="' + params.key + '"\n                            data-autocomplete-data-display="' + params.display + '" >\n                        <span class="glyphicon glyphicon-refresh"></span>\n                      </div>';
    }
    Array.prototype.forEach.call(params.value, function (val) {
      inputSource += (0, _sourceAutocomplete2.default)(val, params);
    });
    inputSource += '</div><input value="" type="text" autocomplete="off" data-value=\'' + lastValues + '\' ' + attributes + ' class="' + inputClass + '" />';
  } else {
    lastValues = JSON.stringify(params.value).replace(/\'/g, '&quote;');
    inputSource += '<select ' + attributes + ' class="' + inputClass + '" last-values=\'' + lastValues + '\'>';

    if (!params.required) inputSource += '<option value=\'\'></option>';
    if (_typeof(params.source) === 'object' && Object.prototype.toString.call(params.source) === '[object Array]') {
      Array.prototype.forEach.call(params.source, function (val) {
        inputSource += (0, _sourceOption2.default)(val, params);
      });
    } else inputSource += (0, _sourceOption2.default)(params.source, params);
    inputSource += '</select>';
  }
  return inputSource;
}

function createInputRich(attributes, inputClass, params) {
  return '<div class="wysiwyg-container rich">\n            <div class="wysiwyg-toolbar wysiwyg-toolbar-top">\n              <a class="wysiwyg-toolbar-icon" href="#" title="Bold (Ctrl+B)" hotkey="b" data-action="bold" data-param="">\n                <span class="glyphicon glyphicon-bold"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Italic (Ctrl+I)" hotkey="i" data-action="italic" data-param="">\n                <span class="glyphicon glyphicon-italic"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Underline (Ctrl+U)" hotkey="u" data-action="underline" data-param="">\n                <span class="glyphicon underline">U</span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Text color" data-action="forecolor" data-param="" data-popup="color">\n                <span class="glyphicon glyphicon-text-color"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Background color" data-action="highlight" data-param="" data-popup="color">\n                <span class="glyphicon glyphicon-text-background"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Left" data-action="align" data-param="left">\n                <span class="glyphicon glyphicon-object-align-left"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Center" data-action="align" data-param="center">\n                <span class="glyphicon glyphicon-object-align-vertical"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Right" data-action="align" data-param="right">\n                <span class="glyphicon glyphicon-object-align-right"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Justify" data-action="justify" data-param="justify">\n                <span class="glyphicon glyphicon-menu-hamburger"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Subscript" data-action="subscript" data-param="">\n                <span class="glyphicon glyphicon-subscript"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Superscript" data-action="superscript" data-param="">\n                <span class="glyphicon glyphicon-superscript"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Indent" data-action="indent" data-param="">\n                <span class="glyphicon glyphicon-triangle-right"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Outdent" data-action="indent" data-param="outdent">\n                <span class="glyphicon glyphicon-triangle-left"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Unordered list" data-action="insertList" data-param="">\n                <span class="glyphicon glyphicon-th-list"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Remove format" data-action="removeFormat" data-param="">\n                <span class="glyphicon glyphicon-remove"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Add link" data-action="insertLink" data-popup="link" data-param="">\n                <span class="glyphicon glyphicon-link"></span>\n              </a>\n              <a class="wysiwyg-toolbar-icon" href="#" title="Code style" data-action="code" data-param="">\n                <span class="glyphicon glyphicon-console"></span>\n              </a>\n            </div>\n            <textarea class="' + inputClass + ' form-rich" ' + attributes + ' rows="4">' + params.value + '</textarea>\n          </div>';
}

function createInputFile(attributes, inputClass, params) {
  return '<input class="form-control" ' + attributes + ' name="' + params.key + '" type="file" />\n          <span class="percent"></span>\n          <input type="text" ' + attributes + ' class="' + inputClass + ' hidden" />';
}

function createInputTextarea(attributes, inputClass, params) {
  return '<textarea class="' + inputClass + '" ' + attributes + ' rows="4">' + params.value + '</textarea>';
}

function createInputLink(attributes, inputClass, params) {
  return '<div class="input-group">\n            <div class="input-group-addon link">\n              <span class="glyphicon glyphicon-link" aria-hidden="true"></span>\n            </div>\n            <input type="text" ' + attributes + ' class="' + inputClass + '" />\n          </div>';
}

function createInputImage(attributes, inputClass, params) {
  return '<div class="input-group img-upload">\n            <div class="input-group-addon image">\n              <span class="glyphicon glyphicon-picture" aria-hidden="true"></span>\n            </div>\n            <input type="text" ' + attributes + ' class="' + inputClass + ' image-input" />\n            <div class="upload-wrapper">\n              <input class="form-control" ' + attributes + ' name="' + params.key + '" type="file" title="upload an image"/>\n              <span class="percent">\n                <span class="glyphicon glyphicon-upload" aria-hidden="true"></span>\n              </span>\n            </div>\n          </div>\n          <div class="input-error"></div>';
}

function createInputText(attributes, inputClass, params) {
  return '<div class="input-group">\n          <div class="input-group-addon">\n            <span class="glyphicon glyphicon-font" aria-hidden="true"></span>\n            </div>\n            <input type="text" ' + attributes + ' class="' + inputClass + '" />\n          </div>';
}

/**
 * Print form input based on input data type {Textarea | text | meta | link | image | ...}
 * && add appropriate attributs / data-attributs
 * @return {String|html} input / input group ...
 */
function printInput(params, root) {
  params = _.abeExtend.hooks.instance.trigger('beforeEditorInput', params);
  var userWorkflow = root.user != null ? root.user.role.workflow : '';
  var res = '<div class="form-group" data-precontrib-templates="' + params.precontribTemplate + '">';
  var inputClass = 'form-control form-abe';
  res += getLabel(params);

  params.placeholder = params.placeholder || '';
  params.value = params.value || '';

  if (typeof params.value === 'string') params.value = params.value.replace(/\"/g, '&quot;');

  params.disabled = '';
  if (params.tab !== 'slug' && !_.User.utils.isUserAllowedOnRoute(userWorkflow, '/abe/operations/' + params.status + '/edit')) {
    params.disabled = 'disabled="disabled"';
  }
  var attributes = getAttributes(params);

  if (params.source != null) {
    params.multiple = (params['max-length'] == null || params['max-length'] > 1) && params.source.length > 0 ? 'multiple' : '';
    params.disabled = params.source.length <= 0 ? 'disabled' : '';
    res += createInputSource(getAttributes(params), inputClass, params);
  } else if (params.type.indexOf('rich') >= 0) res += createInputRich(attributes, inputClass, params);else if (params.type.indexOf('file') >= 0) res += createInputFile(attributes, inputClass, params);else if (params.type.indexOf('textarea') >= 0) res += createInputTextarea(attributes, inputClass, params);else if (params.type.indexOf('link') >= 0) res += createInputLink(attributes, inputClass, params);else if (params.type.indexOf('image') >= 0) res += createInputImage(attributes, inputClass, params);else res += createInputText(attributes, inputClass, params);

  res += '</div>';
  res = _.abeExtend.hooks.instance.trigger('afterEditorInput', res, params);

  return res;
}